﻿# -*- coding: UTF-8 -*-

#   my_math.py      2022.11.23  kita
#!! 数学関係マイモジュール

#import my_math as mmt

import sys
import numpy as np
import matplotlib.pyplot as plt
import math
import decimal
import my_module as mmd


# 少数やマイナスの数値を数値（数字）と判定する関数
#print(mmt.is_num('-1.23'))
# True
def is_num(s):
    try:
        float(s)
    except ValueError:
        return False
    else:
        return True


# マクローリン展開によるsin(x)の近似式を定義
def sin_maclaurin(x, n):
    # 初期化
    y = 0
    # n次近似を計算
    for i in range(n):
        c = (-1)**i / math.factorial(2 * i + 1)
        t = c * x**(2 * i + 1)
        y = y + t
    return y

# マクローリン展開によるarcsin(x)の近似式を定義
def arcsin_maclaurin(x, n):
    # 初期化
    y = 0
    # n次近似を計算
    for i in range(n):
        c = math.factorial(2*i) / 4**i / (math.factorial(i))**2 / (2*i+1)
        t = c * x**(2*i + 1)
        y = y + t
        #print(c, t, y)
    return y

# 作図
def disp_graph(fnc_y, approx_xn, label_f, label_a):
    plt.figure(figsize=(5, 5)) # 画像サイズ
    #plt.figure(figsize=(10, 5)) # 画像サイズ
    plt.plot(x, fnc_y, label=label_f) # 元の関数
    plt.plot(x, approx_xn, label=label_a) # n次近似
    plt.grid() # グリッド線
    plt.legend(prop={"family":"MS Gothic"}) # 凡例
    plt.ylim(-4.0, 4.0) # y軸の表示範囲
    plt.show()

# 指定された桁数の有効数字を返す
def effectiv_number(f0, n, r=0):
    # 数値, 丸める桁数, 丸め方 0:切り捨て　1:四捨五入
    with decimal.localcontext() as ctx:
        ctx.prec = n                        # 有効桁数を設定
        if r == 0:
            ctx.rounding = decimal.ROUND_FLOOR    # 切り捨てを丸め方に設定
        elif r == 1:
            ctx.rounding = decimal.ROUND_HALF_UP    # いわゆる四捨五入を丸め方に設定
        
        f1 = ctx.create_decimal(f0)
    return  float(f1)



#------------------------------------------------------------------------
# startからendまでをdxで区切った数値リストを返す。端数も処理
#   arange2(-9.1234, 3, 0.1, 0)     # [-9.1234,9.1,9.0,...-0.2,-0.1,0]
#   arange2(5, 9.1234, 0.1, 1)      # [5, 5.1,...9, 9.1234]
#------------------------------------------------------------------------
def arange2(start, end, dx, mode=0):   # 0:startを端数処理 1:endを端数処理
    tmp= [];  keta= 10
    if mode == 0: # startを端数処理
        (d,m)= mmd.divmod2(end-start, dx, keta)
        #(ds,ms)= mmd.divmod2(start, dx, keta)
        x1 = start+m
        for i in range(abs(d)+1):
            x= round(x1+i*dx,keta)
            tmp.append(x)
        if m == 0:
            list_x= tmp
        else:
            list_x= [start]+tmp
    #if mode == 0: # startを端数処理
    #    (d,m)= mmd.divmod2(end-start, dx, keta)
    #    (ds,ms)= mmd.divmod2(start, dx, keta)
    #    x1 = ds*dx
    #    for i in range(abs(d)+1):
    #        x= round(x1+i*dx,keta)
    #        tmp.append(x)
    #    if m == 0:
    #        list_x= tmp
    #    else:
    #        list_x= [start]+tmp
         
    elif mode == 1: # endを端数処理
        (d,m)= mmd.divmod2(start-end, dx, keta)
        #(de,m)= mmd.divmod2(end, dx, keta)
        #x1 = de*dx
        for i in range(abs(d)+1):
            x= round(start+i*dx,keta)
            tmp.append(x)
        if m == 0:
            list_x= tmp
        else:
            list_x= tmp+[end]

    return(list_x)

def is_online(p0, p1, p2, gosa= 0.00001):
    if (p0== p1).all() or (p0== p2).all():
        return True 
    (x0, y0)= (p0[0], p0[1])
    (x1, y1)= (p1[0], p1[1])
    (x2, y2)= (p2[0], p2[1])
    if x1== x2: # 垂直線
        if abs(x1-x0) <= gosa:
            return True
        else:
            return False
        
    a= (y1- y2)/(x1- x2)
    b= y1- a*x1
    ye= a*x0+ b
    e= abs(y0- ye)
    print('    is_online 差= %.10f' % e)  ####
    if e<= gosa:
        return True
    else:
        #print('\n   線上点ではありません\007\n')
        return False
    




#--------------------------------------
#   メインルーチン
#--------------------------------------
if __name__ == '__main__':


    if len(sys.argv) < 2:   #引き数無し !!!引数無 !!!argv
        menu = 2
    else:    # 引数１個 
        menu = int(sys.argv[1])
    
    if menu== 9:
        x0, y0= 0.0001, 0.0
        x1, y1= 100.0, 100.0
        x2, y2= 200.0, 200.0
        p0= np.array([x0, y0])
        p1= np.array([x1, y1])
        p2= np.array([x2, y2])
        if is_online(p0, p1, p2):
            print('\n   p0は線上点です')
        else:
            print('\n   p0は線上にありません\007\n')


    # sin(x) のmaclaurin展開
    if menu == 1:
        n = 5  # x軸の値を設定
        x = np.arange(-10.0, 10.0, 0.01)
        # sin(x)を計算
        sin_y = np.sin(x)
        # n次近似を計算
        approx_xn = sin_maclaurin(x, n)
        # Graph
        label_a = '%d 次近似' % n
        disp_graph(sin_y, approx_xn, 'sin(x)', label_a)
    
    # arcsin(x) のmaclaurin展開
    if menu == 2:
        n = 5  # x軸の値を設定
        x = np.arange(-1.0, 1.0, 0.01)
        x[-1] = 1.0
        # arcsin(x)を計算
        arcsin_y = np.arcsin(x)
        # n次近似を計算
        approx_xn = arcsin_maclaurin(x, n)
        # Graph
        label_a = '%d 次近似' % n
        disp_graph(arcsin_y, approx_xn, 'arcsin(x)', label_a)
    
        for i in range(len(x)):
            print('x= %9.5f  近似: %9.5f' % (x[i], approx_xn[i]))
    
    # n次方程式の根を求める
    if menu == 3:
        #a*x^5 + b*x^4 + c*x^3 + d*x^2 + e*x + f = 0
                     # [a,b,c,d,e,f] 
        print(np.roots([1,6,1,0,1,1]))
        # [-5.82410711+0.j, 0.42084104+0.52540961j,  0.42084104-0.52540961j
        #   -0.50878749+0.34645119j, -0.50878749-0.34645119j]



    if menu == 4: # arange2(start, end, dx, mode=1)
        start= 0.0; end= 5.96643; dx= 0.05; mode=1
        list_x1= arange2(start, end,dx, 1)
        print(list_x1)  # 0.0, 0.05, 0.1, 0.15,....5.85, 5.9, 5.95, 5.96643

        start= -end; end= 0.0
        list_x0= arange2(start, end,dx, 0)
        print(list_x0)  # -5.96643, -5.95, -5.9, -5.85...-0.15, -0.1, -0.05, 0.0 

        list_x2= arange2(-5.96643, 8.5, dx, 0)
        print(list_x2)  # -5.96643, -5.95, -5.9, -5.85... 8.4, 8.45, 8.5

    if menu == 5:
        dx= 0.05
        list_x2= arange2(-14.0046696,0, dx, 1)  # end(右端)を端数処理
        print(list_x2)  # 
        list_x3= arange2(0, 14.0046696, dx, 0)  # start(左端)を端数処理
        print(list_x3)  # 
        print(len(list_x2), len(list_x3))

    if menu == 6:
        dx= 0.05
        list_x2= arange2(-14.05,0, dx, 1)  # end(右端)を端数処理
        print(list_x2)  # 
        list_x3= arange2(0, 14.05, dx, 0)  # start(左端)を端数処理
        print(list_x3)  # 
        print(len(list_x2), len(list_x3))

    if menu == 7:
        d_th= 0.05
        end= d_th*170
        list_flw= arange2(0.0, end, d_th, 1)
        print(list_flw); print(len(list_flw))

    if menu== 8:
        th01= 9.300577; th12= 10; th23= 10; th34= 9.300577; d_th= 1
        list_xf= arange2(-(th01+th12), 0.0, d_th, 0)
        list_xb= arange2(0.0, th23+th34, d_th, 1)

        print(len(list_xf), list_xf)
        print(len(list_xb[1:]), list_xb[1:])

